/* --- C ---
**********************************************************************
*
*	Filename		: Xfbrows
*	Description		: XWindow font browser
*
*	Last modified	: 1 April 1990
*	Version			: 2.0
*	Language		: C
*	For machine		: INTERACTIVE Unix
*	Status			: Ready to publish
*	Compile as		: Makefile
*
*	Programed by	: Phade
*	Developed by	: Phade Software
*	Contact			: phade@cs.tu-berlin.de
*
*	Copyright (c) 1991 by Phade Software
*
**********************************************************************
*/

#define MAIN
#include "xfbrows.h"
#include <X11/Xatom.h>

#define QUIT -1
#define FONT "-*-helvetica-medium-r-*-*-12-*"
#define BFONT "-*-helvetica-bold-r-*-*-12-*"

/*	#define FONT "-*-courier-medium-r-*-*-12-*"
	#define BFONT "-*-courier-bold-r-*-*-12-*"	*/

/*	#define FONT "8x13"
	#define BFONT "8x13bold"	*/

static unsigned long rootbg, rootfg;
static char *maingeom = NULL;

static Atom __SWM_VROOT = None;
static char str [128];
static char *def_str;
static int def_int;

static void Syntax ();
static int EventLoop ();
static int rd_int ();
static int rd_str ();
static int rd_flag ();

int main (argc, argv)
int argc;
char *argv [];
{
	int i;
	char *display, *whitestr, *blackstr, *fgstr, *bgstr;
	char *rootfgstr, *rootbgstr;

	XColor ecdef;
	Window rootReturn, parentReturn, *children;
	unsigned int numChildren;

	display = whitestr = blackstr = NULL;
	fgstr = bgstr = rootfgstr = rootbgstr = NULL;
	cmd = rindex (argv [0], '/');
	if (!cmd) cmd = argv [0];
	else cmd++;

	ncols = -1;
	mono = 0;
	bwidth = 2;

	dirW = disW = NULL;
	icongeom = NULL;
	iconic = 0;

	curname = 0;

	for (i=1;i<argc-1;i++)
	{
		if (!strncmp (argv [i], "-d", 2))
		{
			display = argv [++i];
			break;
		}
	}

	if ((theDisp=XOpenDisplay (display)) == NULL)
	{
		fprintf (stderr, "%s: Can't open display\n", argv [0]);
		Quit (-1);
	}

	if (rd_str ("geometry")) maingeom = def_str;
	if (rd_str ("icongeom")) icongeom = def_str;
	if (rd_str ("background")) bgstr = def_str;
	if (rd_str ("foreground")) fgstr = def_str;
	if (rd_int ("borderWidth")) bwidth = def_int;
	iconic = rd_flag ("iconic");
 
	numnames = 0;
	for (i=1;i<argc;i++)
	{
		if (argv [i][0]!= '-') Syntax ();
		else if (!strncmp (argv [i], "-bg", 3))
				{ if (++i<argc) bgstr = argv [i]; }
		else if (!strncmp (argv [i], "-bw", 3))
				{ if (++i<argc) bwidth=atoi (argv [i]); }
		else if (!strncmp (argv [i], "-d", 2))
				{ if (++i<argc) display = argv [i]; }
		else if (!strncmp (argv [i], "-fg", 3))
				{ if (++i<argc) fgstr = argv [i]; }
		else if (!strncmp (argv [i], "-ge", 3))
				{ if (++i<argc) maingeom = argv [i]; }
		else if (!strncmp (argv [i], "-icong", 6))
				{ if (++i<argc) icongeom = argv [i]; }
		else if (!strncmp (argv [i], "-iconi", 6)) iconic = 1;
		else Syntax ();
	}
 
	theScreen = DefaultScreen (theDisp);
	theCmap = DefaultColormap (theDisp, theScreen);
	rootW = RootWindow (theDisp, theScreen);
	theGC = DefaultGC (theDisp, theScreen);
	theVisual = DefaultVisual (theDisp, theScreen);
	ncells = DisplayCells (theDisp, theScreen);
	dispWIDE = DisplayWidth (theDisp, theScreen);
	dispHIGH = DisplayHeight (theDisp, theScreen);
	dispDEEP = DisplayPlanes (theDisp, theScreen);

	__SWM_VROOT = XInternAtom (theDisp, "__SWM_VROOT", False);
	XQueryTree (theDisp, rootW, &rootReturn, &parentReturn, &children, 
		&numChildren);
	for (i = 0;i < numChildren;i++)
	{
		Atom actual_type;
		int actual_format;
		unsigned long nitems, bytesafter;
		Window *newRoot = NULL;
		XWindowAttributes xwa;

		if (XGetWindowProperty (theDisp, children [i], __SWM_VROOT, 0, 1, 
			False, XA_WINDOW, &actual_type, &actual_format, &nitems, 
			&bytesafter, (unsigned char **) &newRoot) == Success && newRoot)
		{
			rootW = *newRoot;
			XGetWindowAttributes (theDisp, rootW, &xwa);
			dispWIDE = xwa.width;dispHIGH = xwa.height;
			dispDEEP = xwa.depth;
			break;
		}
	}

	arrow = XCreateFontCursor (theDisp, XC_top_left_arrow);
	target = XCreateFontCursor (theDisp, XC_leftbutton);
	hand = XCreateFontCursor (theDisp, XC_hand1);

	white = WhitePixel (theDisp, theScreen);
	black = BlackPixel (theDisp, theScreen);
	if (whitestr && XParseColor (theDisp, theCmap, whitestr, &ecdef) &&
		XAllocColor (theDisp, theCmap, &ecdef)) white = ecdef.pixel;
	if (blackstr && XParseColor (theDisp, theCmap, blackstr, &ecdef) &&
		XAllocColor (theDisp, theCmap, &ecdef)) black = ecdef.pixel;

	fg = black;
	bg = white;
	if (fgstr && XParseColor (theDisp, theCmap, fgstr, &ecdef) &&
		XAllocColor (theDisp, theCmap, &ecdef)) fg = ecdef.pixel;
	if (bgstr && XParseColor (theDisp, theCmap, bgstr, &ecdef) &&
		XAllocColor (theDisp, theCmap, &ecdef)) bg = ecdef.pixel;

	rootfg = white;
	rootbg = black;
	if (rootfgstr && XParseColor (theDisp, theCmap, rootfgstr, &ecdef) &&
		XAllocColor (theDisp, theCmap, &ecdef)) rootfg = ecdef.pixel;
	if (rootbgstr && XParseColor (theDisp, theCmap, rootbgstr, &ecdef) &&
		XAllocColor (theDisp, theCmap, &ecdef)) rootbg = ecdef.pixel;

	XSetForeground (theDisp, theGC, fg);
	XSetBackground (theDisp, theGC, bg);

	infofg = fg;
	infobg = bg;

	if (!mono)
	{
		if (theVisual->class == StaticGray || theVisual->class == GrayScale)
		mono = 1;
	}
 
	if ((mfinfo = XLoadQueryFont (theDisp, FONT))==NULL ||
		(bfinfo = XLoadQueryFont (theDisp, BFONT))==NULL)
	{
		sprintf (str,
			"couldn't open the following fonts:\n\t %s \n\t %s", 
			FONT, BFONT);
		FatalError (str);
	}
	mfont=mfinfo->fid;
	bfont=bfinfo->fid;
	SetNormal ();
 
	if (ncols == -1)
	{
		if (dispDEEP>1) ncols = 1<<dispDEEP;
		else ncols = 0;
	}
	else if (ncols>256) ncols = 256;

	CreateDirW (maingeom);

	while ((i=EventLoop ()) != QUIT);
	Quit (0);
	return (0);
}

static void Syntax ()
{
	fprintf (stderr, "usage: %s\n", cmd);
	fprintf (stderr, "   [-display disp] [-iconic]\n");
	fprintf (stderr, "   [-geometry geom] [-icongeom geom]\n");
	fprintf (stderr, "   [-bg background] [-fg foreground] [-bw width]\n");
	Quit (-1);
}

static int EventLoop ()
{
	XEvent event;
	int retval, done;

	done = retval = 0;
	while (!done)
	{
		if (XPending (theDisp)>0)
		{
			XNextEvent (theDisp, &event);
			switch (event.type)
			{
				case Expose:
					{
						XExposeEvent *exp_event = (XExposeEvent *) &event;

						ExposureWin (exp_event);
					}
					break;
				case ButtonPress:
					{
					XButtonEvent *but_event = (XButtonEvent *) &event;
					int i;

					switch (but_event->button)
					{
						case Button1: 
							if (but_event->window == dirW)
							{
								i=ClickDirW (but_event->x, but_event->y);
								if (i==S_BQUIT) { retval = QUIT;done=1;}
							}
							else if (but_event->window == dList.win)
							{
								i=LSClick (&dList, but_event);
								SelectDir (i);
							}
							else if (but_event->window == dList.scrl.win) 
								SCTrack (&dList.scrl, but_event->x,
									but_event->y);
							break;
						default:
							break;
					}
				}
				break;
			case KeyPress:
				{
				XKeyEvent *key_event = (XKeyEvent *) &event;
				char buf [128];
				KeySym ks;
				XComposeStatus status;
				int stlen;
 
				stlen = XLookupString (key_event, buf, 128, &ks, &status);
				if (!stlen) break;
				if (key_event->window == dirW)
					if (DirKey (buf [0])) XBell (theDisp, 0);
				}
				break;
			default:
				break;
			}
		}
	}
	return (retval);
}

/************************************************************************/
/* following three rd_* functions swiped from xgraph, by David Harrison */
/************************************************************************/

static int rd_int (name)
char *name;
{
	if (def_str = XGetDefault (theDisp, PROGNAME, name))
	{
		if (sscanf (def_str, "%ld", &def_int) == 1) return 1;
		else
		{
			fprintf (stderr,
				"%s: couldn't read integer value for %s resource\n", 
				cmd, name);
			return 0;
		}
	}
	else return 0;
}

static int rd_str (name)
char *name;
{
	if (def_str = XGetDefault (theDisp, PROGNAME, name)) return 1;
	else return 0;
}

static int rd_flag (name)
char *name;
{
	if (def_str = XGetDefault (theDisp, PROGNAME, name))
	{
		def_int = (strcmp (def_str, "on")==0) || 
			(strcmp (def_str, "1")==0) ||
			(strcmp (def_str, "true")==0) ||
			(strcmp (def_str, "yes")==0);
		return 1;
	}
	else return 0;
}

